@@ -9,6 +9,7 @@
 #'   path returned by `tools::R_user_dir("rtoot", "config")`.
 #' @param clipboard logical, whether to export the token to the clipboard
 #' @param verbose logical whether to display messages
+#' @param browser if `TRUE` (default) a browser window will be opened to authenticate, else the URL will be provided so you can copy/paste this into the browser yourself
 #' @details If either `name` or `path` are set to `FALSE`, the token is only
 #'   returned and not saved. If you would like to save your token as an environment variable,
 #'   please set `clipboard` to `TRUE`. Your token will be copied to clipboard in the environment variable
@@ -21,7 +22,7 @@
 #' auth_setup("mastodon.social", "public")
 #' }
 #' @export
-auth_setup <- function(instance = NULL, type = NULL, name = NULL, path = NULL, clipboard = FALSE, verbose = TRUE) {
+auth_setup <- function(instance = NULL, type = NULL, name = NULL, path = NULL, clipboard = FALSE, verbose = TRUE, browser = TRUE) {
   while (is.null(instance) || instance == "") {
     instance <- rtoot_ask(prompt = "On which instance do you want to authenticate (e.g., \"mastodon.social\")? ", pass = FALSE)
   }
@@ -29,7 +30,7 @@ auth_setup <- function(instance = NULL, type = NULL, name = NULL, path = NULL, c
   if (!isTRUE(type %in% c("public", "user"))) {
     type <- c("public", "user")[rtoot_menu(choices = c("public", "user"), title = "What type of token do you want?", verbose = TRUE)]
   }
-  token <- process_created_token(create_token(client, type = type), name = name, path = path, clipboard = clipboard, verify = TRUE, verbose = verbose)
+  token <- process_created_token(create_token(client, type = type, browser = browser), name = name, path = path, clipboard = clipboard, verify = TRUE, verbose = verbose)
   return(token) ## explicit
 }
 
@@ -71,10 +72,11 @@ get_client <- function(instance = "mastodon.social"){
 #'
 #' @param client rtoot client object created with [get_client]
 #' @param type one of "public" or "user". See details
+#' @param browser if `TRUE` (default) a browser window will be opened to authenticate, else the URL will be provided so you can copy/paste this into the browser yourself
 #' @details TBA
 #' @return a mastodon bearer token
 #' @references https://docs.joinmastodon.org/client/authorized/
-create_token <- function(client, type = "public"){
+create_token <- function(client, type = "public", browser=TRUE){
   type <- match.arg(type,c("public","user"))
   if(!inherits(client,"rtoot_client")){
     stop("client is not an object of type rtoot_client")
@@ -88,12 +90,17 @@ create_token <- function(client, type = "public"){
       grant_type='client_credentials'
     ))
   } else if(type == "user"){
-    httr::BROWSE(httr::modify_url(url = url, path = "oauth/authorize"),query=list(
+    url <- httr::modify_url(url = url, path = "oauth/authorize")
+    query <- list(
       client_id=client$client_id ,
       redirect_uri='urn:ietf:wg:oauth:2.0:oob',
       scope='read write follow',
-      response_type="code"
-      ))
+      response_type="code")
+    if(browser){
+      httr::BROWSE(url, query=query)
+    }else{
+      message(paste("Navigate to", httr::modify_url(url, query=query), "to obtain an authorization code"))
+    }
     auth_code <- rtoot_ask(prompt = "enter authorization code: ", pass = TRUE, check_rstudio = TRUE, default = "")
     auth2 <- httr::POST(httr::modify_url(url = url, path = "oauth/token"),body=list(
       client_id=client$client_id ,
